==> webmention-0.0.2.tar.gz: webmention-0.0.2/webmention/__init__.py <==
"""
Webmention receiver and sender implementations.

> Webmention is a simple way to notify any URL when you mention it on
> your site. From the receiver's perspective, it's a way to request
> notifications when other sites mention it. 

[0]: https://w3.org/TR/webmention

"""

from understory import sql, web
from understory.web import tx

model = sql.model(
    __name__,
    received_mentions={
        "received": "DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP",
        "mention_id": "TEXT",
        "source": "TEXT",
        "target": "TEXT",
        "data": "JSON",
    },
    sent_mentions={
        "sent": "DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP",
        "mention_id": "TEXT",
        "source": "TEXT",
        "target": "TEXT",
        "response": "JSON",
    },
)


def receive(source, target) -> None:
    """
    Receive a webmention for `target` on behalf of `source`.

    """
    # TODO re-queue failures w/ a logarithmic backoff
    mention_id = store_mention(source, target)
    # mention_data = {}
    source_doc = tx.cache[source]
    source = source_doc.url.rstrip("/")
    mention = source_doc.mention(target).data  # TODO dict(..) inst .data
    tx.db.update(
        "mentions", what="data = ?", where="mention_id = ?", vals=[mention, mention_id]
    )
    return

    # XXX source_data = mf.parse(source_doc.text, url=source)
    # XXX source_repr = mf.util.representative_card(source_data, source)
    # XXX if not source_repr:
    # XXX     raise web.BadRequest("No representative h-card found at source.")

    # XXX ZZZ path = web.uri(target).path
    # XXX ZZZ plural, _, resource_path = path.partition("/")
    # XXX ZZZ try:
    # XXX ZZZ     t = get_type(plural)
    # XXX ZZZ     resource = get_resource(t.s, resource_path)
    # XXX ZZZ except (TypeError, ResourceNotFound):
    # XXX ZZZ     resource = None

    # cache.put(source_url, parse=True)  # , html=source_doc.text)
    # source = interpret(source_url)
    # mention_data["source"] = source

    # # TODO ensure links are in page as with likes below
    # if "entry" in source:
    #     entry = source["entry"]
    #     entry_type = entry["-type"]
    #     if resource:
    #         if entry_type == "photo":
    #             print("photo post")
    #         elif entry_type == "associate":
    #             store("associations", path, source_url, mention_id)
    #         elif entry_type == "like":
    #             for like_of in entry["like-of"]:
    #                 if like_of["url"] == target:
    #                     store("likes", path, source_url, mention_id)
    #         elif entry_type == "vote":
    #             vote_on = entry["vote-on"][0]
    #             if vote_on["url"] == target:
    #                 store("votes", path, source_url, mention_id)
    #         elif entry_type == "note" and t.s == "code":
    #             t.utility(resource["-id"]).add_issue(source_url, entry,
    #                                                  mention_id)
    #         elif entry_type in ("note", "image") and \
    #                                t.s in ("note", "image"):
    #             if "in-reply-to" in entry:
    #                 for in_reply_to in entry["in-reply-to"]:
    #                     if in_reply_to["url"] == target:
    #                         store("replies", path, source_url, mention_id)
    #                 # bubble up mention if resource itself is a reply
    #                 if "in-reply-to" in resource:
    #                     send(path, resource["in-reply-to"])
    #     else:
    #         if entry_type == "follow":
    #             store("follows", path, source_url, mention_id)
    #         elif entry_type == "associate":
    #             store("associations", path, source_url, mention_id)
    # else:
    #     store("generic", path, source_url, mention_id)

    # mention_data["confirmed"] = time.time()
    # tx.kv["mentioned"][mention_id] = pickle.dumps(mention_data)

    # TODO tx.kv.db.publish(f"{tx.host.identity}:resources:{path}",
    # TODO                  f"<section class=h-entry>a {entry_type}</section>")
    #               str(tx.view.entry_template(entry, resource,
    #                                      t.view.type(entry, resource))))

    # # check if networking request
    # source_followees = source_data["rels"].get("followee", [])
    # source_followers = source_data["rels"].get("follower", [])
    # print()
    # print("ees", source_followees)
    # print("ers", source_followers)
    # if "code" in form:
    #     print("code", form.code)
    # print()
    # if "https://{}".format(tx.host.name) in source_followees:
    #     # TODO private webmention for initial follow
    #     # TODO public webmention for public follow
    #     # TODO check first private then public
    #     # TODO represent & reciprocate follow accordingly
    #     root_url = web.uri(source_repr["properties"]["url"][0], secure=True)
    #     person = get_person(root_url.minimized)
    #     rel = tx.db.select("person__person",
    #                        where="from_id = 1 AND to_id = ?",
    #                        vals=[person["id"]])[0]
    #     if rel:
    #         tx.db.update("person__person", what="private = 0",
    #                      where="from_id = 1 AND to_id = ?",
    #                      vals=[person["id"]])
    #         # TODO send return mention to notify if publicize
    #     return "okay"
    #     root_data = mf.parse(url=root_url)
    #     root_repr = mf.util.representative_card(root_data, root_url)
    #     if not root_repr:
    #         raise web.BadRequest("No representative h-card found "
    #                              "at source's root.")
    #     name = root_repr["properties"]["name"][0]
    #     try:
    #         email = root_repr["properties"]["email"][0]
    #     except IndexError:
    #         email = ""
    #     pubkey = ""
    #     person_feed = mf.util.interpret_feed(root_data, source_url)
    #     person_feed_license = root_data["rels"]["license"][0]
    #     add_person(name, root_url, email, pubkey, person_feed["name"],
    #                person_feed_license, "follower")


def send(source, *targets, private=False) -> None:
    """
    Send a webmention to each of `targets` on behalf of `source`.

    """
    # TODO ask archive.org to cache; archive=False
    # TODO re-queue failures w/ a logarithmic backoff
    if source.startswith("/"):
        source = f"{tx.request.uri.scheme}://{tx.request.uri.netloc}/{source}"
    source = str(web.uri(source))
    for target in targets:
        target = str(web.uri(target))
        endpoint = web.discover_link(target, "webmention")
        if not endpoint:
            print(f"no webmention endpoint at {target}")
            return
        payload = {"source": source, "target": target}
        if private:
            code = web.nbrandom(32)
            # TODO tx.kv["mention-tokens"][":".join((path, str(target)))] = code
            payload["code"] = code
        store_mention(source, target)
        web.post(endpoint, data=payload)
        # TODO check if received and/or displayed
        # TODO !!! endpoint.post(payload)
        # TODO tx.db.update("mentions", what="data = ?", where="mention_id = ?",
        # TODO              vals=[mention, mention_id])


def store_mention(source, target):
    mention_id = web.nbrandom(9)
    tx.db.insert("mentions", mention_id=mention_id, source=source, target=target)
    return mention_id


# def store(mention_type, path, source, mention_id) -> bool:
#     """
#     store a webmention
#
#     return True if it was added, false if it already exists
#
#     """
#     return tx.kv["mentions", path, mention_type].hset(source, mention_id)


@model.control
def get_received_mentions(db):
    """Return a list of all received mentions ordered by most recent."""
    return db.select("received_mentions", order="received DESC")


@model.control
def get_received_mention_by_id(db, mention_id):
    return db.select("received_mentions", where="mention_id = ?", vals=[mention_id])


@model.control
def get_received_mentions_by_path(db, url):
    return db.select("received_mentions", where="target = ?", vals=[url])


@model.control
def get_sent_mentions(db):
    """Return a list of all sent mentions ordered by most recent."""
    return db.select("sent_mentions", order="sent DESC")

==> webmention-0.0.3.tar.gz: webmention-0.0.3/webmention/__init__.py <==
"""
Webmention receiver and sender implementations.

> Webmention is a simple way to notify any URL when you mention it on
> your site. From the receiver's perspective, it's a way to request
> notifications when other sites mention it. 

[0]: https://w3.org/TR/webmention

"""

from understory import sql, web
from understory.web import tx

model = sql.model(
    __name__,
    received_mentions={  # others mentioning you
        "mention_id": "TEXT",
        "mentioned": "DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP",
        "source": "TEXT",
        "target": "TEXT",
        "data": "JSON",
    },
    sent_mentions={  # you mentioning others
        "mention_id": "TEXT",
        "mentioned": "DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP",
        "source": "TEXT",
        "target": "TEXT",
        "response": "JSON",
    },
)


def receive(source, target) -> None:
    """
    Receive a webmention for `target` on behalf of `source`.

    """
    # TODO re-queue failures w/ a logarithmic backoff
    mention_id = store_mention(source, target)
    # mention_data = {}
    source_doc = tx.cache[source]
    source = source_doc.url.rstrip("/")
    mention = source_doc.mention(target).data  # TODO dict(..) inst .data
    tx.db.update(
        "mentions", what="data = ?", where="mention_id = ?", vals=[mention, mention_id]
    )
    return

    # XXX source_data = mf.parse(source_doc.text, url=source)
    # XXX source_repr = mf.util.representative_card(source_data, source)
    # XXX if not source_repr:
    # XXX     raise web.BadRequest("No representative h-card found at source.")

    # XXX ZZZ path = web.uri(target).path
    # XXX ZZZ plural, _, resource_path = path.partition("/")
    # XXX ZZZ try:
    # XXX ZZZ     t = get_type(plural)
    # XXX ZZZ     resource = get_resource(t.s, resource_path)
    # XXX ZZZ except (TypeError, ResourceNotFound):
    # XXX ZZZ     resource = None

    # cache.put(source_url, parse=True)  # , html=source_doc.text)
    # source = interpret(source_url)
    # mention_data["source"] = source

    # # TODO ensure links are in page as with likes below
    # if "entry" in source:
    #     entry = source["entry"]
    #     entry_type = entry["-type"]
    #     if resource:
    #         if entry_type == "photo":
    #             print("photo post")
    #         elif entry_type == "associate":
    #             store("associations", path, source_url, mention_id)
    #         elif entry_type == "like":
    #             for like_of in entry["like-of"]:
    #                 if like_of["url"] == target:
    #                     store("likes", path, source_url, mention_id)
    #         elif entry_type == "vote":
    #             vote_on = entry["vote-on"][0]
    #             if vote_on["url"] == target:
    #                 store("votes", path, source_url, mention_id)
    #         elif entry_type == "note" and t.s == "code":
    #             t.utility(resource["-id"]).add_issue(source_url, entry,
    #                                                  mention_id)
    #         elif entry_type in ("note", "image") and \
    #                                t.s in ("note", "image"):
    #             if "in-reply-to" in entry:
    #                 for in_reply_to in entry["in-reply-to"]:
    #                     if in_reply_to["url"] == target:
    #                         store("replies", path, source_url, mention_id)
    #                 # bubble up mention if resource itself is a reply
    #                 if "in-reply-to" in resource:
    #                     send(path, resource["in-reply-to"])
    #     else:
    #         if entry_type == "follow":
    #             store("follows", path, source_url, mention_id)
    #         elif entry_type == "associate":
    #             store("associations", path, source_url, mention_id)
    # else:
    #     store("generic", path, source_url, mention_id)

    # mention_data["confirmed"] = time.time()
    # tx.kv["mentioned"][mention_id] = pickle.dumps(mention_data)

    # TODO tx.kv.db.publish(f"{tx.host.identity}:resources:{path}",
    # TODO                  f"<section class=h-entry>a {entry_type}</section>")
    #               str(tx.view.entry_template(entry, resource,
    #                                      t.view.type(entry, resource))))

    # # check if networking request
    # source_followees = source_data["rels"].get("followee", [])
    # source_followers = source_data["rels"].get("follower", [])
    # print()
    # print("ees", source_followees)
    # print("ers", source_followers)
    # if "code" in form:
    #     print("code", form.code)
    # print()
    # if "https://{}".format(tx.host.name) in source_followees:
    #     # TODO private webmention for initial follow
    #     # TODO public webmention for public follow
    #     # TODO check first private then public
    #     # TODO represent & reciprocate follow accordingly
    #     root_url = web.uri(source_repr["properties"]["url"][0], secure=True)
    #     person = get_person(root_url.minimized)
    #     rel = tx.db.select("person__person",
    #                        where="from_id = 1 AND to_id = ?",
    #                        vals=[person["id"]])[0]
    #     if rel:
    #         tx.db.update("person__person", what="private = 0",
    #                      where="from_id = 1 AND to_id = ?",
    #                      vals=[person["id"]])
    #         # TODO send return mention to notify if publicize
    #     return "okay"
    #     root_data = mf.parse(url=root_url)
    #     root_repr = mf.util.representative_card(root_data, root_url)
    #     if not root_repr:
    #         raise web.BadRequest("No representative h-card found "
    #                              "at source's root.")
    #     name = root_repr["properties"]["name"][0]
    #     try:
    #         email = root_repr["properties"]["email"][0]
    #     except IndexError:
    #         email = ""
    #     pubkey = ""
    #     person_feed = mf.util.interpret_feed(root_data, source_url)
    #     person_feed_license = root_data["rels"]["license"][0]
    #     add_person(name, root_url, email, pubkey, person_feed["name"],
    #                person_feed_license, "follower")


def send(source, *targets, private=False) -> None:
    """
    Send a webmention to each of `targets` on behalf of `source`.

    """
    # TODO ask archive.org to cache; archive=False
    # TODO re-queue failures w/ a logarithmic backoff
    if source.startswith("/"):
        source = f"{tx.request.uri.scheme}://{tx.request.uri.netloc}/{source}"
    source = str(web.uri(source))
    for target in targets:
        target = str(web.uri(target))
        endpoint = web.discover_link(target, "webmention")
        if not endpoint:
            print(f"no webmention endpoint at {target}")
            return
        payload = {"source": source, "target": target}
        if private:
            code = web.nbrandom(32)
            # TODO tx.kv["mention-tokens"][":".join((path, str(target)))] = code
            payload["code"] = code
        store_mention(source, target)
        web.post(endpoint, data=payload)
        # TODO check if received and/or displayed
        # TODO !!! endpoint.post(payload)
        # TODO tx.db.update("mentions", what="data = ?", where="mention_id = ?",
        # TODO              vals=[mention, mention_id])


def store_mention(source, target):
    mention_id = web.nbrandom(9)
    tx.db.insert("mentions", mention_id=mention_id, source=source, target=target)
    return mention_id


# def store(mention_type, path, source, mention_id) -> bool:
#     """
#     store a webmention
#
#     return True if it was added, false if it already exists
#
#     """
#     return tx.kv["mentions", path, mention_type].hset(source, mention_id)


@model.control
def get_received_mentions(db):
    """Return a list of all received mentions ordered by most recent."""
    return db.select("received_mentions", order="received DESC")


@model.control
def get_received_mention_by_id(db, mention_id):
    return db.select("received_mentions", where="mention_id = ?", vals=[mention_id])


@model.control
def get_received_mentions_by_path(db, url):
    return db.select("received_mentions", where="target = ?", vals=[url])


@model.control
def get_sent_mentions(db):
    """Return a list of all sent mentions ordered by most recent."""
    return db.select("sent_mentions", order="sent DESC")
